import 'package:flutter/material.dart';
import 'package:v_flutter/utils/commonColor.dart';

class VCalendar {
  final BuildContext context;
  final Function onTap;
  final String confirmText;
  final String cancel;
  final int period;
  final Map manager;
  final int type;
  final DateTime currentDateInput;

  VCalendar({
    @required this.context,
    this.onTap,
    this.confirmText = '确认',
    this.cancel = '取消',
    this.period = 1,
    this.manager,
    this.type = 1,
    this.currentDateInput,
  }) : assert(context != null) {
    init(context, this);
    monthDayArray = getAllMonthArray();
    insertCalendar(context, this);
  }
  static double dateItemWidth = 30.0;
  static List week = [
    {
      'text': '日',
      'value': 7,
    },
    {
      'text': '一',
      'value': 1,
    },
    {
      'text': '二',
      'value': 2,
    },
    {
      'text': '三',
      'value': 3,
    },
    {
      'text': '四',
      'value': 4,
    },
    {
      'text': '五',
      'value': 5,
    },
    {
      'text': '六',
      'value': 6,
    },
  ];
  static List noonList = [
    {
      'value': '上午',
      'period': 1,
    },
    {
      'value': '下午',
      'period': 2,
    }
  ];
  static OverlayEntry overlayCalendar;

  static DateTime today = DateTime.now();
  static int currentPage = 12;
  static PageController pageController = new PageController(
    initialPage: currentPage,
  );
  static DateTime currentDate = DateTime.now();
  static int currentYear = currentDate.year;
  static int currentMonth = currentDate.month;
  static int currentDay = currentDate.day;
  static int currentYear2 = currentDate.year;
  static int currentMonth2 = currentDate.month;
  static int currentDay2 = currentDate.day;
  static DateTime startDate =
      currentDate.subtract(Duration(days: (365 + currentDay)));
  static DateTime endDate = currentDate.add(Duration(days: 365));

  static List<Map> monthDayArray = [];
  static int resYear = currentYear;
  static int resMonth = currentMonth;
  static int resDay = currentDay;
  static String resNoon = '';
  static int currentPeriod;
  static Map currentManager;
  static void init(BuildContext context, _this) {
    if (_this.currentDateInput != null) {
      currentDate = _this.currentDateInput;
    }
    dateItemWidth = (MediaQuery.of(context).size.width - 52) / 7;
    currentYear = currentDate.year;
    currentMonth = currentDate.month;
    currentDay = currentDate.day;
    resYear = currentYear;
    resMonth = currentMonth;
    resDay = currentDay;
    currentPeriod = _this.period;
    currentManager = _this.manager;
  }

  //type==2头部
  static Widget dateHeader(context, _this) {
    return Container(
      height: 56.0,
      padding: EdgeInsets.only(left: 40.0, right: 40.0),
      child: Flex(
        direction: Axis.horizontal,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[
          GestureDetector(
            onTap: () {
              Navigator.of(context).pop();
            },
            child: Container(
              child: Text(
                "取消",
                style: TextStyle(
                  color: Color(0xFF333333),
                  fontSize: 14.0,
                ),
              ),
            ),
          ),
          Container(
            child: Text(
              '$resYear-$resMonth-$resDay',
              style: TextStyle(
                color: Color(0xFF333333),
                fontSize: 17.0,
              ),
            ),
          ),
          GestureDetector(
            onTap: () {
              _this.onTap({
                "year": resYear,
                "month": resMonth,
                "day": resDay,
                'date': DateTime(resYear, resMonth, resDay),
              });
              Navigator.of(context).pop();
            },
            child: Container(
              child: Text(
                "确定",
                style: TextStyle(
                  color: Color(0xFFF54B64),
                  fontSize: 14.0,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  //日期结果部分
  static Widget dateResult() {
    return Container(
      padding: EdgeInsets.fromLTRB(20.0, 0, 20.0, 0),
      child: Container(
        height: 74,
        alignment: Alignment.center,
        margin: EdgeInsets.only(top: 16.0),
        decoration: BoxDecoration(
          color: VColors.primary,
          borderRadius: BorderRadius.all(
            Radius.circular(10.0),
          ),
        ),
        child: Flex(
          direction: Axis.vertical,
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Text(
              '$resYear-$resMonth-$resDay',
              style: TextStyle(
                color: Color(0xFFFFFFFF),
                fontSize: 22.0,
                letterSpacing: .46,
              ),
            ),
            Text(
              '${currentPeriod == 1 ? '上午' : '下午'}',
              style: TextStyle(
                color: Color(0xFFFFFFFF),
                fontSize: 16.0,
                letterSpacing: .33,
              ),
            ),
          ],
        ),
      ),
    );
  }

  //选择日期部分
  static Widget chooseDateContent(context, state, _this) {
    return Container(
      child: Flex(
        direction: Axis.vertical,
        children: <Widget>[
          chooseDateHead(),
          createDateWrap(context, state),
          _this.type == 1 ? createNoon(state, _this) : Container(),
          _this.type == 1 ? createButton(context, _this) : Container(),
        ],
      ),
    );
  }

  //选择日期头部星期
  static Widget chooseDateHead() {
    List<Widget> weekList = [];
    for (int i = 0; i < week.length; i++) {
      weekList.add(
        Container(
          width: dateItemWidth,
          alignment: Alignment.center,
          margin: EdgeInsets.only(top: 12.0),
          child: Text(
            week[i]['text'],
            style: TextStyle(
              decoration: TextDecoration.none,
              fontSize: 13.0,
              color: Color(0xFFCCCCCC),
            ),
          ),
        ),
      );
    }
    return Container(
      padding: EdgeInsets.fromLTRB(20.0, 0, 20.0, 0),
      child: Flex(
        direction: Axis.horizontal,
        children: weekList,
      ),
    );
  }

  //选择日期一个月中所有日期的包裹层
  static Widget createDateWrap(context, state) {
    return Container(
      height: 240,
      margin: EdgeInsets.only(top: 10),
      padding: EdgeInsets.fromLTRB(20.0, 0, 20.0, 0),
      child: PageView.builder(
        itemBuilder: ((context, index) {
          return Container(
            child: createDateItem(monthDayArray[index], state),
          );
        }),
        itemCount: monthDayArray.length,
        controller: pageController,
        onPageChanged: (index) {
          state(() {
            resYear = monthDayArray[index]['year'];
            resMonth = monthDayArray[index]['month'];
            resDay = 1;
            currentDay = 1;
            currentYear = monthDayArray[index]['year'];
            currentMonth = monthDayArray[index]['month'];
          });
        },
      ),
    );
  }

  //选择日期每个日期
  static Widget createDateItem(date, state) {
    List<Widget> dateList = [];
    for (int i = 0; i < 42; i++) {
      dateList.add(
        i > date['firstDayWeek'] - 1 &&
                i < date['daysLen'] + date['firstDayWeek']
            ? GestureDetector(
                onTap: () {
                  state(() {
                    resDay = i - date['firstDayWeek'] + 1;
                    currentDay = i - date['firstDayWeek'] + 1;
                    currentYear = resYear;
                    currentMonth = resMonth;
                  });
                },
                child: (date['year'] == currentYear &&
                        date['month'] == currentMonth &&
                        i - date['firstDayWeek'] + 1 == currentDay)
                    ? Container(
                        width: dateItemWidth,
                        height: 40.0,
                        alignment: Alignment.center,
                        child: Container(
                          width: 32.0,
                          height: 32.0,
                          alignment: Alignment.center,
                          decoration: BoxDecoration(
                              borderRadius: BorderRadius.all(
                                Radius.circular(12.0),
                              ),
                              border: Border.all(
                                color: VColors.primary,
                              )),
                          child: Text(
                            '${i - date['firstDayWeek'] + 1}',
                            style: TextStyle(
                              decoration: TextDecoration.none,
                              fontSize: 12.0,
                              color: Color(0xFF333333),
                            ),
                          ),
                        ),
                      )
                    : Container(
                        width: dateItemWidth,
                        height: 40.0,
                        color: Color.fromRGBO(0, 0, 0, 0),
                        alignment: Alignment.center,
                        child: Container(
                          child: Text(
                            '${i - date['firstDayWeek'] + 1}',
                            style: TextStyle(
                              decoration: TextDecoration.none,
                              fontSize: 14.0,
                              color: Color(0xFF333333),
                            ),
                          ),
                        ),
                      ),
              )
            : Container(
                width: dateItemWidth,
                height: 40.0,
              ),
      );
    }
    return Wrap(
      children: dateList,
    );
  }

  //选择当日的上午
  static Widget createNoon(state, _this) {
    List<Widget> list = [];
    for (var item in noonList) {
      list.add(
        GestureDetector(
          onTap: () {
            state(() {
              currentPeriod = item['period'];
            });
          },
          child: Container(
            width: 101.0,
            height: 48.0,
            margin: EdgeInsets.only(left: 16.0),
            alignment: Alignment.center,
            decoration: BoxDecoration(
              color: VColors.gray,
              borderRadius: BorderRadius.all(
                Radius.circular(10.0),
              ),
              border: Border.all(
                color: currentPeriod == item['period']
                    ? VColors.primary
                    : VColors.gray,
              ),
            ),
            child: Text(
              '${item['value']}',
              style: TextStyle(
                color: VColors.text,
                fontSize: 13.0,
              ),
            ),
          ),
        ),
      );
    }
    return Container(
      padding: EdgeInsets.fromLTRB(20.0, 0, 20.0, 0),
      child: Flex(
        direction: Axis.horizontal,
        mainAxisAlignment: MainAxisAlignment.center,
        children: list,
      ),
    );
  }

  static createButton(context, _this) {
    return Container(
      height: 52.0,
      margin: EdgeInsets.only(top: 25.0),
      decoration: BoxDecoration(
        border: Border(
          top: BorderSide(
            color: VColors.gray,
          ),
        ),
      ),
      child: Flex(
        direction: Axis.horizontal,
        children: <Widget>[
          Expanded(
            child: GestureDetector(
              onTap: () {
                Navigator.of(context).pop();
              },
              child: Container(
                alignment: Alignment.center,
                decoration: BoxDecoration(
                  border: Border(
                    right: BorderSide(
                      color: VColors.gray,
                    ),
                  ),
                ),
                child: Text(
                  '${_this.cancel}',
                  style: TextStyle(
                    fontSize: 18.0,
                    color: VColors.text,
                  ),
                ),
              ),
            ),
          ),
          Expanded(
            child: GestureDetector(
              onTap: () {
                _this.onTap({
                  'date': DateTime(resYear, resMonth, resDay),
                  'period': currentPeriod,
                  'manager': currentManager,
                });
                Navigator.of(context).pop();
              },
              child: Container(
                alignment: Alignment.center,
                child: Text(
                  '${_this.confirmText}',
                  style: TextStyle(
                    fontSize: 18.0,
                    color: VColors.primary,
                  ),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  static removeCalendar() {
    // overlayCalendar.remove();
  }

  //月份相关函数方法
  static int getMonthDaysLen(year, month) {
    if (month == 2 && year % 4 != 0) {
      return 28;
    } else if (month == 2 && year % 4 == 0) {
      if (year % 100 > 0) {
        return 29;
      } else {
        if (year % 400 == 0) {
          return 29;
        }
        return 28;
      }
    } else if ([1, 3, 5, 7, 8, 10, 12].contains(month)) {
      return 31;
    } else {
      return 30;
    }
  }

  static List<Map> getAllMonthArray() {
    List<Map> monthArr = [];
    int monthDaysLen = 0;
    int allDays = 0;
    while (startDate.add(Duration(days: allDays)).isBefore(endDate)) {
      monthDaysLen = getMonthDaysLen(
          startDate.add(Duration(days: allDays)).year,
          startDate.add(Duration(days: allDays)).month);

      monthArr.add({
        'year': startDate.add(Duration(days: allDays)).year,
        'month': startDate.add(Duration(days: allDays)).month,
        'daysLen': monthDaysLen,
        'firstDayWeek': DateTime(startDate.add(Duration(days: allDays)).year,
                        startDate.add(Duration(days: allDays)).month)
                    .weekday ==
                7
            ? 0
            : DateTime(startDate.add(Duration(days: allDays)).year,
                    startDate.add(Duration(days: allDays)).month)
                .weekday,
      });
      allDays += monthDaysLen;
    }
    return monthArr;
  }

  static insertCalendar(BuildContext context, _this) {
    showModalBottomSheet(
        context: context,
        isScrollControlled: true,
        backgroundColor: Color.fromRGBO(0, 0, 0, 0),
        builder: (BuildContext context) {
          return StatefulBuilder(
            builder: (context1, state) {
              return Container(
                height: _this.type == 1 ? 512 : 350,
                decoration: BoxDecoration(
                  color: Color(0xFFFFFFFF),
                  borderRadius: BorderRadius.only(
                    topLeft: Radius.circular(5.0),
                    topRight: Radius.circular(5.0),
                  ),
                ),
                child: Flex(
                  direction: Axis.vertical,
                  children: <Widget>[
                    _this.type == 1 ? dateResult() : dateHeader(context, _this),
                    chooseDateContent(context, state, _this),
                  ],
                ),
              );
            },
          );
        });
  }
}
