import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/animation.dart';

class VConfirm {
  static Animation<double> animationScale;
  static AnimationController animationScaleController;
  static Animation<double> animationOpacity;
  static AnimationController animationOpacityController;
  static Duration animateTime = Duration(milliseconds: 400);
  final BuildContext context;
  final String message;
  final String title;
  final String confirmText;
  final String cancelText;
  final Function onCancel;
  final Function onConfirm;
  VConfirm({
    @required this.context,
    @required this.message,
    this.title,
    this.confirmText,
    this.cancelText,
    this.onCancel,
    this.onConfirm,
  })  : assert(context != null),
        assert(message != null) {
    _show();
  }
  _show() async {
    animationOpacityController = AnimationController(
      duration: animateTime,
      vsync: Overlay.of(context),
    );
    animationScaleController = AnimationController(
      duration: animateTime,
      vsync: Overlay.of(context),
    );
    animationOpacity = Tween(begin: 0.0, end: 1.0).animate(
      CurvedAnimation(parent: animationOpacityController, curve: Curves.ease),
    );
    animationScale = Tween(begin: 0.0, end: 1.0).animate(
      CurvedAnimation(parent: animationOpacityController, curve: Curves.ease),
    );
    showDialog(
      context: context,
      builder: (context) => Container(
        child: Center(
          child: AnimatedBuilder(
            animation: animationOpacity,
            child: Container(
              decoration: BoxDecoration(
                color: Color(0xFFFFFFFF),
                borderRadius: BorderRadius.all(
                  Radius.circular(6),
                ),
              ),
              width: 270,
              child: Flex(
                direction: Axis.vertical,
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Container(
                    padding: EdgeInsets.only(top: 13),
                    child: Text(
                      title,
                      style: TextStyle(
                        fontSize: 18,
                        fontWeight: FontWeight.w500,
                        decoration: TextDecoration.none,
                        color: Color(0xFF000000),
                      ),
                    ),
                  ),
                  Container(
                    margin: EdgeInsets.only(top: 5),
                    padding: EdgeInsets.only(left: 13, right: 13),
                    child: Text(
                      message,
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        fontSize: 14,
                        fontWeight: FontWeight.w500,
                        decoration: TextDecoration.none,
                        color: Color(0xFF000000),
                      ),
                    ),
                  ),
                  Container(
                    margin: EdgeInsets.only(top: 13),
                    decoration: BoxDecoration(
                      border: Border(
                        top: BorderSide(
                          color: Color.fromRGBO(0, 0, 0, .2),
                          width: .5,
                        ),
                      ),
                    ),
                    height: 44,
                    child: Flex(
                      direction: Axis.horizontal,
                      children: <Widget>[
                        Expanded(
                          child: Material(
                            color: Color(0xFFFFFFFF),
                            borderRadius: BorderRadius.only(
                              bottomLeft: Radius.circular(6),
                            ),
                            child: Ink(
                              child: InkWell(
                                onTap: () {
                                  confirm(onConfirm, context);
                                },
                                child: Container(
                                  alignment: Alignment.center,
                                  decoration: BoxDecoration(
                                    border: Border(
                                      right: BorderSide(
                                        color: Color.fromRGBO(0, 0, 0, .2),
                                        width: .5,
                                      ),
                                    ),
                                  ),
                                  child: Text(
                                    confirmText,
                                    style: TextStyle(
                                      fontSize: 18,
                                      fontWeight: FontWeight.w500,
                                      decoration: TextDecoration.none,
                                      color: Color(0xFF007aff),
                                    ),
                                  ),
                                ),
                              ),
                            ),
                          ),
                        ),
                        Expanded(
                          child: Material(
                            color: Color(0xFFFFFFFF),
                            borderRadius: BorderRadius.only(
                              bottomRight: Radius.circular(6),
                            ),
                            child: Ink(
                              child: InkWell(
                                onTap: () {
                                  cancel(onCancel, context);
                                },
                                child: Container(
                                  alignment: Alignment.center,
                                  child: Text(
                                    cancelText,
                                    style: TextStyle(
                                      fontSize: 18,
                                      fontWeight: FontWeight.w500,
                                      decoration: TextDecoration.none,
                                      color: Color(0xFF007aff),
                                    ),
                                  ),
                                ),
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  )
                ],
              ),
            ),
            builder: (context, buildChild) {
              return Opacity(
                opacity: animationOpacity.value,
                child: AnimatedBuilder(
                  animation: animationScale,
                  builder: (context, _) {
                    return Transform.scale(
                      scale: animationScale.value,
                      child: buildChild,
                    );
                  },
                ),
              );
            },
          ),
        ),
      ),
    );
    animationOpacityController.forward();
    animationScaleController.forward();
  }

  static cancel(func, context) async {
    if (func != null) {
      func();
    }
    animationOpacityController.reverse();
    animationScaleController.reverse();
    await Future.delayed(animateTime);
    Navigator.of(context).pop();
  }

  static confirm(func, context) async {
    if (func != null) {
      func();
    }
    animationOpacityController.reverse();
    animationScaleController.reverse();
    await Future.delayed(animateTime);
    Navigator.of(context).pop();
  }
}
