/*
 * @Description: 
 * @version: 1.0.0
 * @Author: youqixin
 * @Date: 2022-02-15 14:35:02
 * @LastEditors: youqixin
 * @LastEditTime: 2022-02-17 14:25:52
 */
import 'package:flutter/material.dart';
import 'package:v_flutter/utils/commonColor.dart';

class VPicker {
  int currentIndex = 0;
  bool isSelfScorll = true;
  ScrollController _controller = ScrollController();
  static const double stepHeight = 48.0;
  static const int emptyCont = 3;
  static double listHeight = (2 * emptyCont + 1) * stepHeight;
  final BuildContext context;
  final String title;
  final String confirmText;
  final String cancelText;
  final Function onCancel;
  final Function onConfirm;
  final List<Map> options;
  VPicker({
    @required this.context,
    this.title = "",
    this.confirmText = "确定",
    this.cancelText = "取消",
    this.onCancel,
    this.onConfirm,
    this.options = const [],
  }) : assert(context != null) {
    _show();
  }
  _show() {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      backgroundColor: Color.fromRGBO(0, 0, 0, 0),
      builder: (BuildContext context) {
        return StatefulBuilder(
          builder: (context1, state) {
            return Container(
              height: 500,
              color: VColors.white,
              child: Flex(
                direction: Axis.vertical,
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  Container(
                    height: 40,
                    padding: EdgeInsets.only(left: 12.0, right: 12.0),
                    child: Flex(
                      direction: Axis.horizontal,
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: <Widget>[
                        GestureDetector(
                          onTap: () {
                            if (onCancel != null) {
                              onCancel();
                            }
                            Navigator.of(context).pop();
                          },
                          child: Text(
                            cancelText,
                            style: TextStyle(
                              fontSize: 16.0,
                              color: VColors.text,
                            ),
                          ),
                        ),
                        Text(
                          title,
                          style: TextStyle(
                            fontSize: 18.0,
                            color: VColors.text,
                          ),
                        ),
                        GestureDetector(
                          onTap: () {
                            Map resJson = options[currentIndex];
                            String res = resJson["value"];
                            if (onConfirm != null) {
                              onConfirm(res);
                            }
                            Navigator.of(context).pop();
                          },
                          child: Text(
                            confirmText,
                            style: TextStyle(
                              fontSize: 16.0,
                              color: VColors.primary,
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                  Container(
                    height: listHeight,
                    child: Stack(
                      alignment: Alignment.center,
                      children: <Widget>[
                        Positioned(
                          top: emptyCont * stepHeight,
                          left: 0,
                          child: Container(
                            padding: EdgeInsets.only(left: 12.0, right: 12.0),
                            width: MediaQuery.of(context).size.width,
                            child: Container(
                              height: stepHeight,
                              decoration: BoxDecoration(
                                border: Border(
                                  top: BorderSide(
                                    color: VColors.border,
                                    width: 1.0,
                                    style: BorderStyle.solid,
                                  ),
                                  bottom: BorderSide(
                                    color: VColors.border,
                                    width: 1.0,
                                    style: BorderStyle.solid,
                                  ),
                                ),
                              ),
                            ),
                          ),
                        ),
                        Positioned(
                          top: 0,
                          left: 0,
                          child: NotificationListener<ScrollNotification>(
                            onNotification: (ScrollNotification notification) {
                              if (isSelfScorll) {
                                double distance = notification.metrics.pixels;
                                int index =
                                    (distance / stepHeight - 0.5).ceil();
                                state(() {
                                  currentIndex = index;
                                });
                                if (notification is ScrollEndNotification) {
                                  isSelfScorll = false;
                                  Future.delayed(
                                    Duration(microseconds: 0),
                                    () {
                                      _controller.jumpTo(index * stepHeight);
                                      isSelfScorll = true;
                                    },
                                  );
                                }
                              }
                              return false;
                            },
                            child: Container(
                              height: listHeight,
                              width: MediaQuery.of(context).size.width,
                              child: MediaQuery.removePadding(
                                context: context,
                                removeTop: true,
                                removeBottom: true,
                                child: ListView(
                                  controller: _controller,
                                  shrinkWrap: true,
                                  children: _createList(),
                                ),
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            );
          },
        );
      },
    );
  }

  List<Widget> _createList() {
    List<Widget> list = [];
    for (int i = 0, len = emptyCont; i < len; i++) {
      list.add(
        Container(
          height: stepHeight,
          padding: EdgeInsets.only(left: 12.0, right: 12.0),
          alignment: Alignment(0, 0),
        ),
      );
    }
    for (int i = 0, len = options.length; i < len; i++) {
      list.add(
        Container(
          height: stepHeight,
          padding: EdgeInsets.only(left: 12.0, right: 12.0),
          alignment: Alignment(0, 0),
          child: Text(
            options[i]["label"],
            style: TextStyle(
              color: i == currentIndex ? VColors.active : VColors.text,
              fontSize: i == currentIndex ? 18 : 16,
            ),
          ),
        ),
      );
    }
    for (int i = 0, len = emptyCont; i < len; i++) {
      list.add(
        Container(
          height: stepHeight,
          padding: EdgeInsets.only(left: 12.0, right: 12.0),
          alignment: Alignment(0, 0),
        ),
      );
    }
    return list;
  }
}
