import 'dart:async';
import 'package:flutter/animation.dart';
import 'package:flutter/material.dart';

enum ToastPosition {
  TOP,
  CENTER,
  BOTTOM,
}

class VToast {
  static OverlayEntry overlayView;
  static DateTime _startedTime;
  static Duration animateTime = Duration(milliseconds: 400);

  final BuildContext context;
  final String message;
  final Duration duration;
  final ToastPosition position;
  VToast({
    @required this.context,
    @required this.message,
    this.duration = const Duration(milliseconds: 3000),
    this.position = ToastPosition.CENTER,
  })  : assert(context != null),
        assert(message != null) {
    _show();
  }

  _show() async {
    overlayView?.remove();
    overlayView = null;
    _startedTime = DateTime.now();
    Animation<double> animationDown;
    Animation<double> animationOpactity;
    AnimationController animationControllerDown;
    AnimationController animationControllerOpactity;
    animationControllerDown = AnimationController(
      duration: animateTime,
      vsync: Overlay.of(context),
    );
    animationControllerOpactity = AnimationController(
      duration: animateTime,
      vsync: Overlay.of(context),
    );
    animationDown = position == ToastPosition.BOTTOM
        ? Tween(
                begin: _getPosition(position, context) + 100.0,
                end: _getPosition(position, context))
            .animate(
            CurvedAnimation(
                parent: animationControllerDown, curve: Curves.ease),
          )
        : Tween(
                begin: _getPosition(position, context) - 100.0,
                end: _getPosition(position, context))
            .animate(
            CurvedAnimation(
                parent: animationControllerDown, curve: Curves.ease),
          );
    animationOpactity = Tween(begin: 0.0, end: 1.0).animate(
      CurvedAnimation(parent: animationControllerOpactity, curve: Curves.ease),
    );
    overlayView = OverlayEntry(
      builder: (context) {
        return _fadeInDownToast(animationDown, animationOpactity, message);
      },
    );
    Overlay.of(context).insert(overlayView);
    animationControllerDown.forward();
    animationControllerOpactity.forward();
    await Future.delayed(duration);
    if (_startedTime != null &&
        DateTime.now().difference(_startedTime) >= duration) {
      animationControllerDown.reverse();
      animationControllerOpactity.reverse();
      await Future.delayed(animateTime);
      overlayView?.remove();
      overlayView = null;
    }
    return true;
  }

  static Widget _fadeInDownToast(animationDown, animationOpactity, message) {
    return AnimatedBuilder(
      animation: animationDown,
      child: Container(
        decoration: BoxDecoration(
          color: Color(0xFF323232),
          borderRadius: BorderRadius.all(Radius.circular(6)),
        ),
        child: Padding(
          padding: EdgeInsets.symmetric(horizontal: 12.0, vertical: 8.0),
          child: Text(
            message,
            style: TextStyle(
              fontSize: 14,
              fontWeight: FontWeight.w500,
              decoration: TextDecoration.none,
              color: Color(0xFFFFFFFF),
            ),
          ),
        ),
      ),
      builder: (context, childBuilder) {
        return Positioned(
          top: animationDown.value,
          child: Container(
            alignment: Alignment.center,
            width: MediaQuery.of(context).size.width,
            child: AnimatedBuilder(
              animation: animationOpactity,
              builder: (context, _) {
                return Opacity(
                  opacity: animationOpactity.value,
                  child: childBuilder,
                );
              },
            ),
          ),
        );
      },
    );
  }

  static double _getPosition(position, context) {
    switch (position) {
      case ToastPosition.TOP:
        return MediaQuery.of(context).size.height * 1 / 6;
        break;
      case ToastPosition.CENTER:
        return MediaQuery.of(context).size.height / 2;
        break;
      default:
        return MediaQuery.of(context).size.height * 5 / 6;
        break;
    }
  }

  static void cancel() {
    overlayView?.remove();
    overlayView = null;
    _startedTime = null;
  }
}
